# -*- coding: utf-8 -*-
from unittest import TestCase
from jsondb.db import Database
from jsondb.compat import u


class BaseTestCase(TestCase):

    def setUp(self):
        import json

        content = {
            "url": "http://sky.net",
            "ip": "http://10.0.1.337",
        }

        with open("test.db", "w+") as test:
            json.dump(content, test)

        self.database = Database("test.db")

    def tearDown(self):
        import os

        os.remove("test.db")


class Tests(BaseTestCase):

    def test_create_none_exists(self):
        """
        A file should be created if none exists
        and an empty {} is placed in the file.
        """
        from os import path
        import os

        database = Database("new.db")

        data = ""
        with open ("new.db", "r") as db:
            data = db.read()

        self.assertTrue(path.exists("new.db"))
        self.assertTrue("{}" in data)

        os.remove("new.db")

    def test_empty_file_initialized(self):
        """
        Test that when a file exists but is empty,
        a {} is added.
        """
        import os

        # Create an new empty file
        open("empty.db", 'a').close()

        database = Database("empty.db")

        data = ""
        with open ("empty.db", "r") as db:
            data = db.read()

        self.assertTrue("{}" in data)

        os.remove("empty.db")

    def test_get_key(self):
        key = self.database.data(key="url")

        self.assertEqual(key, "http://sky.net")

    def test_get_key_not_in_database(self):
        key = self.database.data(key="bogus")

        self.assertEqual(key, None)

    def test_assign_key_value_pair(self):
        self.database.data(key="cool", value="robot")

        self.assertEqual(self.database.data(key="cool"), "robot")

    def test_assign_dictionary(self):
        d = {
            "id": "123456",
            "arduino_ip": "xxxxxx"
        }
        self.database.data(dictionary=d)

        self.assertTrue("id" in self.database.data())
        self.assertEqual(self.database.data(key="arduino_ip"), "xxxxxx")

    def test_delete_key(self):
        self.database.data(key="id", value="13")
        self.database.delete("id")

        self.assertFalse("id" in self.database.data())


class UnicodeTests(BaseTestCase):

    def test_data_with_unicode_key(self):

        unicode_text = u("∰ ∱ ∲ ∳ ⨋ ⨌⨔ ⨕ ⨖ ⨗ ⨘ ⨙ ⨚ ∫ ∬ ∭ ∮ ∯  ⨍ ⨎ ⨏ ⨐ ⨑ ⨒ ⨓  ⨛ ⨜")

        self.database.data(key=unicode_text, value=13)

        self.assertTrue(unicode_text in self.database.data())
        self.assertEqual(self.database.data(key=unicode_text), 13)

    def test_data_with_unicode_value(self):

        unicode_text = u("∠ ∡ ⦛ ⦞ ⦟ ⦢ ⦣ ⦤ ⦥ ⦦ ⦧ ⦨ ⦩ ⦪ ⦫ ⦬ ⦭ ⦮ ⦯ ⦓ ⦔ ⦕ ⦖ ⟀")

        self.database.data(key="unicode", value=unicode_text)

        self.assertEqual(unicode_text, self.database.data(key="unicode"))


class DictCompatibleTests(BaseTestCase):

    def test_get_data(self):
        data = self.database["url"]

        self.assertEqual(data, "http://sky.net")

    def test_get_unset_data(self):
        data = self.database["invalid"]

        self.assertEqual(data, None)

    def test_set_data(self):
        self.database["new"] = "test"

        self.assertEqual(self.database["new"], "test")

    def test_key_exists(self):
        self.assertTrue("url" in self.database)
        self.assertFalse("invalid" in self.database)

    def test_delete_data(self):
        del self.database["url"]

        self.assertEqual(self.database["url"], None)


class UtilityTests(TestCase):

    def test_nonexistant_database_is_invalid(self):
        from jsondb.file_writer import is_valid

        self.assertFalse(is_valid("some_nonexistant_file.db"))



'''
Use the faster cjson library if it is available
'''

try:
    import cjson as json

    json_encode = json.encode
    json_decode = json.decode

except ImportError:
    import json

    json_encode = json.dumps
    json_decode = json.loads


def encode(value):
    return json_encode(value)

def decode(value):
    return json_decode(value)

'''
Python 2 and 3 unicode string compatability.
'''

import sys
if sys.version < "3":
    import codecs
    def u(x):
        return codecs.unicode_escape_decode(x)[0]
else:
    def u(x):
        return x



# -*- coding: utf-8 -*-
from .file_writer import read_data, write_data, is_valid


class Database(object):

    def __init__(self, file_path):
        """
        This class manages a json-formatted file database.
        Constructor takes the file path of the database as a parameter.
        """
        self.path = None
        self.set_path(file_path)

    def set_path(self, file_path):
        """
        Set the path of the database.
        Create the file if it does not exist.
        """
        if not is_valid(file_path):
            write_data(file_path, {})

        self.path = file_path

    def _get_content(self, key=None):
        obj = read_data(self.path)

        if key:
            if key in obj.keys():
                return obj[key]
            else:
                return None

        return obj

    def _set_content(self, key, value):
        obj = self._get_content()
        obj[key] = value

        data = write_data(self.path, obj)

    def delete(self, key):
        """
        Removes the specified key from the database.
        """
        obj = self._get_content()
        obj.pop(key, None)

        data = write_data(self.path, obj)

    def data(self, **kwargs):
        """
        If a key is passed in, a corresponding value will be returned.
        If a key-value pair is passed in then the corresponding key in
        the database will be set to the specified value.
        A dictionary can be passed in as well.
        If a key does not exist and a value is provided then an entry
        will be created in the database.
        """

        key = kwargs.pop('key', None)
        value = kwargs.pop('value', None)
        dictionary = kwargs.pop('dictionary', None)

        # Fail if a key and a dictionary or a value and a dictionary are given
        if (key is not None and dictionary is not None) or \
           (value is not None and dictionary is not None):
            raise ValueError

        # If only a key was provided return the corresponding value
        if key is not None and value is None:
            return self._get_content(key)

        # if a key and a value are passed in
        if key is not None and value is not None:
            self._set_content(key, value)

        if dictionary is not None:
            for key in dictionary.keys():
                value = dictionary[key]
                self._set_content(key, value)

        return self._get_content()

    # Iterator methods

    def __len__(self):
        return len(self._get_content())

    def __iter__(self):
        return iter(self._get_content().keys())

    def __list__(self):
        return list(self._get_content().keys())

    # Dictionary compatibility methods

    def __getitem__(self, key):
        return self.data(key=key)

    def __setitem__(self, key, value):
        return self.data(key=key, value=value)

    def __delitem__(self, key):
        return self.delete(key=key)

    def __contains__(self, key):
        return key in self._get_content()



from .compat import decode, encode


def read_data(path):
    """
    Reads a file and returns a json encoded representation of the file.
    """
    db = open(path, "r+")
    content = db.read()

    obj = decode(content)

    db.close()

    return obj

def write_data(path, obj):
    """
    Writes to a file and returns the updated file content.
    """
    with open(path, "w+") as db:
        db.write(encode(obj))

    return obj

def is_valid(file_path):
    """
    Check to see if a file exists or is empty
    """
    from os import path, stat

    return path.exists(file_path) and stat(file_path).st_size > 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

history = open("history.rst").read().replace(".. :changelog:", "")

readme = lambda f: open(f, "r").read()

req = open("requirements.txt")
requirements = req.readlines()

setup(
    name="jsondatabase",
    version="0.0.6",
    description="A flat file database for json objects.",
    long_description=readme("readme.md") + "\n\n" + history,
    author="Gunther Cox",
    author_email="gunthercx@gmail.com",
    url="https://github.com/gunthercox/jsondb",
    packages=["jsondb"],
    package_dir={"jsondb": "jsondb"},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords = ["jsondb"],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
    ],
    test_suite="tests",
    tests_require=[]
)
